public class SimpleThreads {

    static void threadMessage(String message) {
	String threadName = Thread.currentThread().getName();
	System.out.format("%s: %s%n", threadName, message);
    }

    private static class MessageLoop implements Runnable {
          public void run() {
	      String info[] = {
		  "The problem with the future",
		  "is that it",
		  "keeps turning",
		  "into present" };
	      try {
		  for (int i = 0; i < info.length; i++) {
		      Thread.sleep(10000);
		      threadMessage(info[i]);
		  }
	      }
	      catch (InterruptedException e) {
		  threadMessage("I wasn't done");
	      }
	  }
    }

    public static void main (String args[]) 
	throws InterruptedException
    {
	// tempo de espera em ms antes de interromper MessageLoop
	long patience = 1000 * 60 * 60; // default - uma hora

	if (args.length > 0) {
	    try {
		patience = Long.parseLong(args[0]) *1000;
	    } catch (NumberFormatException e) {
		System.err.println("Argument must be an integer");
		System.exit(1);
	    }
	}

	// captura o tempo atual do sistema
	long startTime = System.currentTimeMillis();

	// cria um objeto MessageLoop com referencia Thread
	Thread t = new Thread(new MessageLoop());
	threadMessage("Starting MessageLoop thread in main");
	// passa a thread para o estado 'runnable'
	t.start();

	threadMessage("Waiting for 'MessageLoop' thread to finish");

	while (t.isAlive()) {
	  threadMessage("Still waiting ... ");
	  // espera no maximo 1 segundo para 't' finalizar
	  t.join(1000);
	  if (((System.currentTimeMillis()-startTime) > patience) &&
	       t.isAlive()) {
	      threadMessage("Tired of waiting");
	      t.interrupt();
	      // aguarda indefinidamente ate que 't' finalize 
	      // (agora nao deve demorar)
	      t.join();
	  }
	}
	threadMessage("Finally!");
    }
} 
